@@ -7,12 +7,16 @@
  *   there is no warranty.
  *
  *   faith
- *   1.1.1.1
- *   1995/02/22 19:09:24
+ *   1.7
+ *   1995/06/04 04:20:25
  *
  */
 
+static char rcsId[] = "$Version: chfn.c,v 1.7 1995/06/04 04:20:25 faith Exp $";
+
+#if 0
 #define _POSIX_SOURCE 1
+#endif
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -59,8 +63,10 @@ static int check_gecos_string P((char *msg, char *gecos));
 static boolean set_changed_data P((struct finfo *oldfp, struct finfo *newfp));
 static int save_new_data P((struct finfo *pinfo));
 static void *xmalloc P((int bytes));
+#if 0
 extern int strcasecmp P((char *, char *));
 extern int setpwnam P((struct passwd *pwd));
+#endif
 #define memzero(ptr, size) memset((char *) ptr, 0, size)
 
 int main (argc, argv)
@@ -117,7 +123,7 @@ int main (argc, argv)
 	perror (whoami);
 	return (-1);
     }
-    
+
     if (interactive) ask_info (&oldf, &newf);
 
     if (! set_changed_data (&oldf, &newf)) {
