@@ -15,6 +15,7 @@
 #include <ctype.h>
 #include <time.h>
 #include <string.h>
+#include <errno.h>
 
 extern int is_local(char *);
 
@@ -23,6 +24,14 @@ extern int is_local(char *);
 
 #define MAX_LENGTH	1024
 
+void
+pexit(str)
+     char *str;
+{
+    perror(str);
+    exit(1);
+}
+
 int
 main(argc, argv)
 	int argc;
@@ -38,6 +47,7 @@ main(argc, argv)
 	char salt[2];
 	FILE *fd_in, *fd_out;
 	char line[MAX_LENGTH];
+	char colonuser[16];
 	int error=0;
 	int r;
 
@@ -90,12 +100,9 @@ main(argc, argv)
 	
 redo_it:
 	pwdstr = getpass("Enter new password: ");
-	strncpy(pwdstr1, pwdstr, 9);
-	pwdstr = getpass("Re-type new password: ");
-
-	if(strncmp(pwdstr, pwdstr1, 8)) {
-		puts("You misspelled it. Password not changed.");
-		exit(0);
+	if (pwdstr[0] == '\0') {
+	   puts("Password not changed.");
+	   exit(0);
 	}
 	
 	if((strlen(pwdstr) < 6) && gotuid) {
@@ -139,6 +146,14 @@ redo_it:
 
 	/* do various other checks for stupid passwords here... */
 
+	strncpy(pwdstr1, pwdstr, 9);
+	pwdstr = getpass("Re-type new password: ");
+
+	if(strncmp(pwdstr, pwdstr1, 8)) {
+		puts("You misspelled it. Password not changed.");
+		exit(0);
+	}
+	
 	time(&tm);
 	salt[0] = bin_to_ascii(tm & 0x3f);
 	salt[1] = bin_to_ascii((tm >> 5) & 0x3f);
@@ -158,8 +173,11 @@ redo_it:
 		puts("Can't read /etc/passwd, can't update password");
 		exit(1);
 	}
+
+	strcpy(colonuser, user);
+	strcat(colonuser, ":");
 	while(fgets(line, sizeof(line), fd_in)) {
-		if(!strncmp(line,user,strlen(user))) {
+		if(!strncmp(line,colonuser,strlen(colonuser))) {
 			pe->pw_passwd = cryptstr;
 			if(putpwent(pe, fd_out) < 0) {
 				error = 1;
@@ -180,11 +198,17 @@ redo_it:
 	fclose(fd_in);
 	fclose(fd_out);
 
-	unlink("/etc/passwd.OLD");
-	link("/etc/passwd", "/etc/passwd.OLD");
-	unlink("/etc/passwd");
-	link("/etc/ptmp", "/etc/passwd");
-	unlink("/etc/ptmp");
+	if (unlink("/etc/passwd.OLD") < 0)
+	  pexit("unlink(/etc/passwd.OLD) failed: no change");
+	if (link("/etc/passwd", "/etc/passwd.OLD")) 
+	  pexit("link(/etc/passwd, /etc/passwd.OLD) failed: no change");
+	if (unlink("/etc/passwd") < 0)
+	  pexit("unlink(/etc/passwd) failed: no change");
+	if (link("/etc/ptmp", "/etc/passwd") < 0)
+	  pexit("link(/etc/ptmp, /etc/passwd) failed: PASSWD file DROPPED!!");
+	if (unlink("/etc/ptmp") < 0) 
+	  pexit("unlink(/etc/ptmp) failed: /etc/ptmp still exists");
+
 	chmod("/etc/passwd", 0644);
 	chown("/etc/passwd", 0, 0);
 
