@@ -39,7 +39,7 @@ char copyright[] =
 
 #ifndef lint
 /*static char sccsid[] = "from: @(#)vipw.c	5.16 (Berkeley) 3/3/91";*/
-static char rcsid[] = "vipw.c,v 1.1.1.1 1995/02/22 19:09:25 faith Exp";
+static char rcsid[] = "vipw.c,v 1.2 1995/03/12 01:33:05 faith Exp";
 #endif /* not lint */
 
 #include <sys/types.h>
