@@ -47,6 +47,7 @@
  *   [ -standout [attr] ]
  *   [ -msg [on|off] ]
  *   [ -msglevel [0-8] ]
+ *   [ -powersave [on|off] ]
  *
  *
  * Semantics:
@@ -135,7 +136,7 @@ int opt_background, opt_bold, opt_blink, opt_reverse, opt_underline;
 int opt_store, opt_clear, opt_blank, opt_snap, opt_snapfile, opt_standout;
 int opt_append, opt_ulcolor, opt_hbcolor, opt_halfbright, opt_repeat;
 int opt_tabs, opt_clrtabs, opt_regtabs, opt_appcursorkeys, opt_inversescreen;
-int opt_msg, opt_msglevel;
+int opt_msg, opt_msglevel, opt_powersave;
 
 /* Option controls.  The variable names have been contracted to ensure
  * uniqueness.
@@ -143,7 +144,7 @@ int opt_msg, opt_msglevel;
 char *opt_te_terminal_name;	/* Terminal name. */
 int opt_cu_on, opt_li_on, opt_sn_on, opt_so_on, opt_bo_on, opt_hb_on, opt_bl_on;
 int opt_re_on, opt_un_on, opt_rep_on, opt_appck_on, opt_invsc_on;
-int opt_msg_on;		/* Boolean switches. */
+int opt_msg_on, opt_ps_on;	/* Boolean switches. */
 int opt_ke_type;		/* Keyboard type. */
 int opt_fo_color, opt_ba_color;	/* Colors. */
 int opt_ul_color, opt_hb_color;
@@ -610,6 +611,8 @@ int *bad_arg;			/* Set to true if an error is detected. */
 	parse_switch(argc, argv, &opt_msg, &opt_msg_on, bad_arg);
   else if (STRCMP(option, "msglevel") == 0)
 	parse_msglevel(argc, argv, &opt_msglevel, &opt_msglevel_num, bad_arg);
+  else if (STRCMP(option, "powersave") == 0)
+	parse_switch(argc, argv, &opt_powersave, &opt_ps_on, bad_arg);
 #if 0
   else if (STRCMP(option, "standout") == 0)
 	parse_standout(argc, argv, &opt_standout, &opt_st_attr, bad_arg);
@@ -673,17 +676,11 @@ char *prog_name;		/* Name of this program. */
   fprintf(stderr, "  [ -file dumpfilename ]\n");
   fprintf(stderr, "  [ -msg [on|off] ]\n");
   fprintf(stderr, "  [ -msglevel [0-8] ]\n");
+  fprintf(stderr, "  [ -powersave [on|off] ]\n");
 }
 
 char tc_ent_buf[TC_ENT_SIZE];	/* Buffer for storing a termcap entry. */
 
-#define NUM_COLS 160 
-#define NUM_ROWS 75
-
-/* made unsigned, per Ken Jordan (kenjordan@bix.com). --faith@cs.unc.edu,
-Sat Feb 4 20:21:58 1995 */
-unsigned char screenbuf[NUM_ROWS*NUM_COLS];
-
 char *tc_entry(name)
 char *name;			/* Termcap capability string to lookup. */
 {
@@ -948,6 +945,15 @@ int vcterm;			/* Set if terminal is a virtual console. */
   if (opt_blank) 
     printf("\033[9;%d]", opt_bl_min);
     
+  /* -powersave [on|off] (console) */
+  if (opt_powersave) {
+        char ioctlarg[2];
+	ioctlarg[0] = 10;	/* powersave */
+	ioctlarg[1] = opt_ps_on;
+        if (ioctl(0,TIOCLINUX,ioctlarg))
+	    fprintf(stderr,"cannot (un)set powersave mode\n");
+  }
+
 #if 0
   /* -standout [num]. */
   if (opt_standout)
@@ -955,36 +961,18 @@ int vcterm;			/* Set if terminal is a virtual console. */
 #endif
 
   /* -snap [1-NR_CONS]. */
-  if (opt_snap || opt_append) 
-  { FILE *F;
-    char buf[NUM_COLS+1];
-    int i, j;
-    if (opt_snap)
-      F = fopen(opt_sn_name,"w");
-    else  
-      F = fopen(opt_sn_name,"a");
-    if (!F)
-    { fprintf(stderr,"can not open dump file %s for output\n",opt_sn_name); 
-      exit(-1);
-    }
-    screenbuf[0] = 0;
-    screenbuf[1] = (unsigned char)opt_sn_num;
-    if (ioctl(0,TIOCLINUX,screenbuf)>=0)
-    { int rows, cols;
-      rows = (int)screenbuf[0];
-      cols = (int)screenbuf[1];
-      for (i=0; i<rows; i++)
-      { strncpy(buf, screenbuf+2+(cols*i),cols);
-        buf[cols] = '\0';
-        j = cols;
-        while (--j && (buf[j] == ' '))
-          buf[j] = '\0';
-        fputs(buf,F);
-        fputc('\n',F); 
-      }  
-    }
-    else
-      fprintf(stderr,"can not ioctl dump\n");
+  if (opt_snap || opt_append) {
+      FILE *F;
+
+      F = fopen(opt_sn_name, opt_snap ? "w" : "a");
+      if (!F) {
+	  perror(opt_sn_name);
+	  fprintf(stderr,"setterm: can not open dump file %s for output\n",
+		  opt_sn_name); 
+	  exit(-1);
+      }
+      screendump(opt_sn_num, F);
+      fclose(F);
   }
 
   /* -msg [on|off]. */
@@ -1009,6 +997,78 @@ int vcterm;			/* Set if terminal is a virtual console. */
    }
 }
 
+extern char *malloc();
+
+screendump(int vcnum, FILE *F){
+#include <sys/param.h>
+    char infile[MAXPATHLEN];
+    unsigned char header[4];
+    unsigned int rows, cols;
+    int fd, i, j;
+    char *inbuf, *outbuf, *p, *q;
+
+    sprintf(infile, "/dev/vcsa%d", vcnum);
+    fd = open(infile, 0);
+    if (fd < 0 || read(fd, header, 4) != 4)
+      goto try_ioctl;
+    rows = header[0];
+    cols = header[1];
+    if (rows * cols == 0)
+      goto try_ioctl;
+    inbuf = malloc(rows*cols*2);
+    outbuf = malloc(rows*(cols+1));
+    if(!inbuf || !outbuf) {
+	fprintf(stderr, "Out of memory\n");
+	exit(1);
+    }
+    if (read(fd, inbuf, rows*cols*2) != rows*cols*2) {
+	fprintf(stderr, "Error reading %s\n", infile);
+	exit(1);
+    }
+    p = inbuf;
+    q = outbuf;
+    for(i=0; i<rows; i++) {
+	for(j=0; j<cols; j++) {
+	    *q++ = *p;
+	    p += 2;
+	}
+	while(j-- > 0 && q[-1] == ' ')
+	  q--;
+	*q++ = '\n';
+    }
+    if (fwrite(outbuf, 1, q-outbuf, F) != q-outbuf) {
+	fprintf(stderr, "Error writing screendump\n");
+	exit(1);
+    }
+    return;
+
+try_ioctl:
+    {
+#define NUM_COLS 160
+#define NUM_ROWS 75
+	char buf[NUM_COLS+1];
+	unsigned char screenbuf[NUM_ROWS*NUM_COLS];
+	screenbuf[0] = 0;
+	screenbuf[1] = (unsigned char) vcnum;
+	if (ioctl(0,TIOCLINUX,screenbuf) < 0) {
+	    fprintf(stderr,"couldn't read %s, and cannot ioctl dump\n",
+		    infile);
+	    exit(1);
+	}
+	rows = screenbuf[0];
+	cols = screenbuf[1];
+	for (i=0; i<rows; i++) {
+	    strncpy(buf, screenbuf+2+(cols*i), cols);
+	    buf[cols] = '\0';
+	    j = cols;
+	    while (--j && (buf[j] == ' '))
+	      buf[j] = '\0';
+	    fputs(buf,F);
+	    fputc('\n',F); 
+	}
+    }
+}
+
 void main(int argc, char **argv)
 {
   int bad_arg = FALSE;		/* Set if error in arguments. */
@@ -1067,7 +1127,7 @@ void main(int argc, char **argv)
 
   /* See if the terminal is a virtual console terminal. */
 
-  vcterm = !strncmp(term, "con", 3) || !strncmp(term, "linux", 5);
+  vcterm = (!strncmp(term, "con", 3) || !strncmp(term, "linux", 5));
 
   /* Perform the selected options. */
 
