@@ -20,6 +20,7 @@
  * checked.
  *
  * Wed Feb  8 12:27:00 1995: Andries.Brouwer@cwi.nl fixed up error messages.
+ * Sat Jun  3 20:44:38 1995: Patches from Andries.Brouwer@cwi.nl applied.
  *
  */
 
@@ -33,6 +34,9 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#define PROC_FILESYSTEMS	"/proc/filesystems"
+
+
 int del_loop (const char *);
 
 /* True for fake mount (-f).  */
@@ -99,15 +103,21 @@ const struct opt_map opt_map[] =
 #ifdef MS_NOSUB
   { "sub",	1, MS_NOSUB	},	/* allow submounts */
   { "nosub",	0, MS_NOSUB	},	/* don't allow submounts */
+#endif
+#ifdef  MS_SILENT
+  { "quiet",    0, MS_SILENT    },      /* be quiet  */
+  { "loud",     1, MS_SILENT    },      /* print out messages. */
 #endif
   { NULL,	0, 0		}
 };
 
+int mount_quiet=0;
 
 /* Report on a single mount.  */
 static void
 print_one (const struct mntent *mnt)
 {
+  if (mount_quiet) return;
   printf ("%s on %s", mnt->mnt_fsname, mnt->mnt_dir);
   if ((mnt->mnt_type != NULL) && *mnt->mnt_type != '\0')
     printf (" type %s", mnt->mnt_type);
@@ -151,6 +161,12 @@ parse_opt (const char *opt, int *mask, char *extra_opts)
 	  *mask |= om->mask;
 	if (om->mask == MS_USER)
 	  *mask |= MS_SECURE;
+#ifdef MS_SILENT
+        if (om->mask == MS_SILENT && om->inv)  {
+          mount_quiet = 1;
+          verbose = 0;
+        }
+#endif
 	return;
       }
   if (*extra_opts)
@@ -290,8 +306,12 @@ try_mount5 (char *spec, char *node, char **type, int flags, char *mount_opts) {
    char line[100];
    char fsname[50];
    
-   if (*type) return mount5 (spec, node, *type, flags & ~MS_NOSYS, mount_opts);
-   if (( procfs_file = fopen("/proc/filesystems", "r")) == NULL) {
+   if (strcasecmp (*type, "auto") == 0)
+      *type = NULL;
+
+   if (*type)
+      return mount5 (spec, node, *type, flags & ~MS_NOSYS, mount_opts);
+   if (( procfs_file = fopen(PROC_FILESYSTEMS, "r")) == NULL) {
 				/* If /proc/filesystems is not available,
 				   preserve the old behavior of mount. */
       return mount5 (spec,
@@ -303,9 +323,9 @@ try_mount5 (char *spec, char *node, char **type, int flags, char *mount_opts) {
       if (sscanf (line, "nodev %[^\n]\n", fsname) == 1) continue;
       if (sscanf (line, " %[^ \n]\n", fsname) != 1) continue;
       if (mount5 (spec, node, fsname, flags & ~MS_NOSYS, mount_opts) == 0) {
-      *type=xstrdup(fsname);
-      fclose(procfs_file);
-      return 0;
+	 *type=xstrdup(fsname);
+	 fclose(procfs_file);
+	 return 0;
       }
    }
    fclose(procfs_file);
@@ -364,7 +384,10 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
 #endif
 
   if (!type && !(type = fstype(spec)))
-  	return 1;
+    {
+      error ("mount: you must specify a filesystem type for %s", spec);
+      return 1;
+    }
 
   block_signals (SIG_BLOCK);
 
@@ -443,7 +466,15 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
     case EIO:
       error ("mount: %s: can't read superblock", spec); break;
     case ENODEV:
-      error ("mount: fs type %s not supported by kernel", type); break;
+      error ("mount: fs type %s not supported by kernel", type);
+      { FILE *pfs = fopen(PROC_FILESYSTEMS, "r");
+	if (pfs != NULL) {
+	  fclose (pfs);
+	  error ("mount: [for the supported filesystem types, see %s]",
+		 PROC_FILESYSTEMS);
+        }
+      }
+      break;
     case ENOTBLK:
       error ("mount: %s is not a block device", spec); break;
     case ENXIO:
@@ -460,7 +491,7 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
          anti_recurse++;
          if (opts)
            {
-             opts = realloc(xstrdup(opts), strlen(opts)+3);
+             opts = realloc(xstrdup(opts), strlen(opts)+4);
              strcat(opts, ",ro");
            }
          else
@@ -514,6 +545,7 @@ mount_all (string_list types)
       }
 
   status = 0;
+  if (!setfsent()) return 1;
   while ((fstab = getfsent ()) != NULL)
     if (matching_type (fstab->mnt_type, types)
 	 && !streq (fstab->mnt_dir, "/")
@@ -592,6 +624,7 @@ static void
 usage (FILE *fp, int n)
 {
   fprintf (fp, "%s", usage_string);
+  unlock_mtab();
   exit (n);
 }
 
