@@ -29,7 +29,20 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- */
+ *
+ * Wed Sep 14 21:56:59 1994: Applied patches from Alan Modra
+ * (alan@spri.levels.unisa.edu.au):
+ * 1) Add O_CREAT to open flags so that syslogd doesn't complain about
+ *    non-existent files
+ * 2) Modified f_pmask initialisation and testing to allow logging of
+ *    messages at a particular priority level, rather that all messages
+ *    at or above a given priority level.
+ *
+ * Sat Jun 3 12:48:16 1995: Applied patches from
+ * Jochen.Hein@informatik.tu-clausthal.de to allow spaces *AND* tabs to
+ * separate the selector from the action. This means that no whitespace is
+ * allowed inside the selector.
+ * */
 
 #ifndef lint
 char copyright[] =
@@ -79,6 +92,8 @@ static char sccsid[] = "@(#)syslogd.c	5.45 (Berkeley) 3/2/91";
  * Rick <pclink@qus102.qld.npb.telecom.com.au>.  Anyone else
  * named Rick who wants to chip in?  :-)
  * More corrections by Neal Becker <neal@ctd.comsat.com> Sun Jan 16, 1994
+ * Patches from Jochen Hein (Jochen.Hein@informatik.tu-clausthal.de) to treat
+ *   spaces and tabs the same, applied Sat Mar 11 10:09:24 1995
  */
 
 #ifdef __linux__
@@ -133,7 +148,18 @@ static char sccsid[] = "@(#)syslogd.c	5.45 (Berkeley) 3/2/91";
 
 char	*LogName = _PATH_LOG;
 char	*ConfFile = _PATH_LOGCONF;
+
+#ifdef __linux__
+#define CONFORM_TO_FSSTND
+#include "pathnames.h"
+#endif
+
+#ifdef CONFORM_TO_FSSTND
+char	*PidFile = "/var/run/syslog.pid";
+#else
 char	*PidFile = _PATH_LOGPID;
+#endif
+
 char	ctty[] = _PATH_CONSOLE;
 
 #define FDMASK(fd)	(1 << (fd))
@@ -644,7 +670,7 @@ logmsg(pri, msg, from, flags)
 		fac = LOG_NFACILITIES;
 	else
 		fac = LOG_FAC(pri);
-	prilev = LOG_PRI(pri);
+	prilev = 1 << (LOG_PRIMASK - LOG_PRI(pri));
 
 	/* log the message to the particular outputs */
 	if (!Initialized) {
@@ -664,8 +690,7 @@ logmsg(pri, msg, from, flags)
 	}
 	for (f = Files; f; f = f->f_next) {
 		/* skip messages that are incorrect priority */
-		if (f->f_pmask[fac] < prilev ||
-		    f->f_pmask[fac] == INTERNAL_NOPRI)
+		if (!(f->f_pmask[fac] & prilev))
 			continue;
 
 		if (f->f_type == F_CONSOLE && (flags & IGN_CONS))
@@ -1092,10 +1117,10 @@ init()
 	if (Debug) {
 		for (f = Files; f; f = f->f_next) {
 			for (i = 0; i <= LOG_NFACILITIES; i++)
-				if (f->f_pmask[i] == INTERNAL_NOPRI)
+				if (f->f_pmask[i] == 0)
 					printf("X ");
 				else
-					printf("%d ", f->f_pmask[i]);
+					printf("0x%02x ", f->f_pmask[i]);
 			printf("%s: ", TypeNames[f->f_type]);
 			switch (f->f_type) {
 			case F_FILE:
@@ -1147,45 +1172,53 @@ cfline(line, f)
 	/* clear out file entry */
 	bzero((char *) f, sizeof *f);
 	for (i = 0; i <= LOG_NFACILITIES; i++)
-		f->f_pmask[i] = INTERNAL_NOPRI;
+		f->f_pmask[i] = 0;
 
 	/* scan through the list of selectors */
-	for (p = line; *p && *p != '\t';) {
+	for (p = line; *p && *p != '\t' && *p != ' ';) {
 
 		/* find the end of this facility name list */
-		for (q = p; *q && *q != '\t' && *q++ != '.'; )
+		for (q = p; *q && *q != '\t' && *q != ' ' && *q++ != '.'; )
 			continue;
 
 		/* collect priority name */
-		for (bp = buf; *q && !index("\t,;", *q); )
+		for (bp = buf; *q && !index("\t,; ", *q); )
 			*bp++ = *q++;
 		*bp = '\0';
 
 		/* skip cruft */
-		while (index(", ;", *q))
+		while (index(",;", *q)) /* spaces no longer allowed! */
 			q++;
 
 		/* decode priority name */
 		if (*buf == '*')
-			pri = LOG_PRIMASK + 1;
+			pri = -1;
 		else {
-			pri = decode(buf, prioritynames);
-			if (pri < 0) {
+                        pri = decode(*buf == '=' ? buf+1 : buf, prioritynames);
+                        if (pri == INTERNAL_NOPRI)
+                                pri = 0;
+                        else if (pri < 0 || pri > LOG_PRIMASK) {
 				(void) sprintf(ebuf,
 				    "unknown priority name \"%s\"", buf);
 				logerror(ebuf);
 				return;
 			}
+			else
+                                pri = (*buf == '=' ? 1 : -1) << (LOG_PRIMASK - pri);
+			      
 		}
 
 		/* scan facilities */
-		while (*p && !index("\t.;", *p)) {
-			for (bp = buf; *p && !index("\t,;.", *p); )
+		while (*p && !index("\t.; ", *p)) {
+			for (bp = buf; *p && !index("\t,;. ", *p); )
 				*bp++ = *p++;
 			*bp = '\0';
 			if (*buf == '*')
 				for (i = 0; i < LOG_NFACILITIES; i++)
-					f->f_pmask[i] = pri;
+                                        if (pri == 0)
+                                                f->f_pmask[i] = pri;
+                                        else
+                                                f->f_pmask[i] |= pri;
 			else {
 				i = decode(buf, facilitynames);
 				if (i < 0) {
@@ -1195,7 +1228,10 @@ cfline(line, f)
 					logerror(ebuf);
 					return;
 				}
-				f->f_pmask[i >> 3] = pri;
+                                if (pri == 0)
+                                        f->f_pmask[i >> 3] = pri;
+                                else
+                                       f->f_pmask[i >> 3] |= pri;
 			}
 			while (*p == ',' || *p == ' ')
 				p++;
@@ -1205,7 +1241,7 @@ cfline(line, f)
 	}
 
 	/* skip to action part */
-	while (*p == '\t')
+	while (*p == '\t' || *p == ' ')
 		p++;
 
 	switch (*p)
@@ -1234,9 +1270,12 @@ cfline(line, f)
 	case '/':
 		(void) strcpy(f->f_un.f_fname, p);
 #ifdef O_NOCTTY
-		if ((f->f_file = open(p, O_WRONLY|O_APPEND|O_NOCTTY, 0)) < 0) {
+                if ((f->f_file = open(p,
+				      O_CREAT|O_WRONLY|O_APPEND|O_NOCTTY,
+				      0644)) < 0) {
 #else /* !O_NOCTTY */
-		if ((f->f_file = open(p, O_WRONLY|O_APPEND, 0)) < 0) {
+                if ((f->f_file = open(p,
+				      O_CREAT|O_WRONLY|O_APPEND, 0644)) < 0) {
 #endif /* !O_NOCTTY */
 			f->f_file = F_UNUSED;
 			logerror(p);
