@@ -45,7 +45,7 @@ char copyright[] =
 
 #ifndef lint
 /*static char sccsid[] = "from: @(#)rev.c	5.2 (Berkeley) 3/21/92";*/
-static char rcsid[] = "rev.c,v 1.1.1.1 1995/02/22 19:09:19 faith Exp";
+static char rcsid[] = "rev.c,v 1.2 1995/03/12 01:36:02 faith Exp";
 #endif /* not lint */
 
 #include <sys/types.h>
